/* $OpenLDAP: pkg/ldap/include/avl.h,v 1.16 1999/11/27 22:40:08 hyc Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */
/* Portions
 * Copyright (c) 1993 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */
/* avl.h - avl tree definitions */


#ifndef _AVL
#define _AVL

#include <port_cdefs.h>

/*
 * this structure represents a generic avl tree node.
 */

PT_BEGIN_DECL

typedef struct avlnode Avlnode;

#ifdef AVL_INTERNAL
struct avlnode {
	void*		avl_data;
	signed int		avl_bf;
	struct avlnode	*avl_left;
	struct avlnode	*avl_right;
};

#define NULLAVL	((Avlnode *) NULL)

/* balance factor values */
#define LH 	(-1)
#define EH 	0
#define RH 	1

/* avl routines */
#define avl_getone(x)	((x) == 0 ? 0 : (x)->avl_data)
#define avl_onenode(x)	((x) == 0 || ((x)->avl_left == 0 && (x)->avl_right == 0))

#endif /* AVL_INTERNALS */

typedef int		(*AVL_APPLY) PT_P((void *, void*));
typedef int		(*AVL_CMP) PT_P((const void*, const void*));
typedef int		(*AVL_DUP) PT_P((void*, void*));
typedef void	(*AVL_FREE) PT_P((void*));

LIBAVL_F( int )
avl_free PT_P(( Avlnode *root, AVL_FREE dfree ));

LIBAVL_F( int )
avl_insert PT_P((Avlnode **, void*, AVL_CMP, AVL_DUP));

LIBAVL_F( void* )
avl_delete PT_P((Avlnode **, void*, AVL_CMP));

LIBAVL_F( void* )
avl_find PT_P((Avlnode *, const void*, AVL_CMP));

LIBAVL_F( void* )
avl_find_lin PT_P((Avlnode *, const void*, AVL_CMP));

#ifdef AVL_NONREENTRANT
LIBAVL_F( void* )
avl_getfirst PT_P((Avlnode *));

LIBAVL_F( void* )
avl_getnext PT_P((void));
#endif

LIBAVL_F( int )
avl_dup_error PT_P((void*, void*));

LIBAVL_F( int )
avl_dup_ok PT_P((void*, void*));

LIBAVL_F( int )
avl_apply PT_P((Avlnode *, AVL_APPLY, void*, int, int));

LIBAVL_F( int )
avl_prefixapply PT_P((Avlnode *, void*, AVL_CMP, void*, AVL_CMP, void*, int));

/* apply traversal types */
#define AVL_PREORDER	1
#define AVL_INORDER	2
#define AVL_POSTORDER	3
/* what apply returns if it ran out of nodes */
#define AVL_NOMORE	(-6)

PT_END_DECL

#endif /* _AVL */
