/* $OpenLDAP: pkg/ldap/include/lutil.h,v 1.25 1999/12/13 03:54:00 kdz Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */

#ifndef _LUTIL_H
#define _LUTIL_H 1

#include <port_cdefs.h>

/*
 * Include file for PortThreads utility routine
 */

PORT_BEGIN_DECL

/* n octets encode into ceiling(n/3) * 4 bytes */
/* Avoid floating point math by through extra padding */

#define LUTIL_BASE64_ENCODE_LEN(n)	((n)/3 * 4 + 4)
#define LUTIL_BASE64_DECODE_LEN(n)	((n)/4 * 3)

/* ISC Base64 Routines */
/* base64.c */

LIBLUTIL_F( int )
lutil_b64_ntop PORT_P((
	unsigned char const *,
	size_t,
	char *,
	size_t));

LIBLUTIL_F( int )
lutil_b64_pton PORT_P((
	char const *,
	unsigned char *,
	size_t));

/* detach.c */
LIBLUTIL_F( void )
lutil_detach PORT_P((
	int debug,
	int do_close));

/* entropy.c */
LIBLUTIL_F( int )
lutil_entropy PORT_P((
	char *buf,
	ber_len_t nbytes ));

/* passwd.c */
struct berval; /* avoid pulling in lber.h */

LIBLUTIL_F( int )
lutil_passwd PORT_P((
	const struct berval *passwd,	/* stored password */
	const struct berval *cred,	/* user supplied value */
	const char **methods ));

LIBLUTIL_F( struct berval * )
lutil_passwd_generate PORT_P(( ber_len_t ));

LIBLUTIL_F( struct berval * )
lutil_passwd_hash PORT_P((
	const struct berval *passwd,
	const char *method ));

LIBLUTIL_F( int )
lutil_passwd_scheme PORT_P((
	const char *scheme ));

/* utils.c */
LIBLUTIL_F( char* )
lutil_progname PORT_P((
	const char* name,
	int argc,
	char *argv[] ));

/* sockpair.c */
LIBLUTIL_F( int )
lutil_pair( LBER_SOCKET_T sd[2] );

PORT_END_DECL

#endif /* _LUTIL_H */
