/* $OpenLDAP: pkg/ldap/include/lutil_sha1.h,v 1.16 1999/12/08 05:44:23 kdz Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */

/* This version is based on:
 *	$OpenBSD: sha1.h,v 1.8 1997/07/15 01:54:23 millert Exp $	*/

#ifndef _LUTIL_SHA1_H_
#define _LUTIL_SHA1_H_

#include <port_cdefs.h>
#include <ac/bytes.h>

PORT_BEGIN_DECL

/*
 * SHA-1 in C
 * By Steve Reid <steve@edmweb.com>
 */
#define LUTIL_SHA1_BYTES 20

/* This code assumes char are 8-bits and uint32 are 32-bits */
typedef ac_uint4 uint32;

typedef struct {
    uint32 state[5];
    uint32 count[2];  
    unsigned char buffer[64];
} lutil_SHA1_CTX;
  
LIBLUTIL_F( void )
lutil_SHA1Transform
	PORT_P((uint32 state[5], const unsigned char buffer[64]));

LIBLUTIL_F( void  )
lutil_SHA1Init
	PORT_P((lutil_SHA1_CTX *context));

LIBLUTIL_F( void  )
lutil_SHA1Update
	PORT_P((lutil_SHA1_CTX *context, const unsigned char *data, uint32 len));

LIBLUTIL_F( void  )
lutil_SHA1Final
	PORT_P((unsigned char digest[20], lutil_SHA1_CTX *context));

LIBLUTIL_F( char * )
lutil_SHA1End
	PORT_P((lutil_SHA1_CTX *, char *));

LIBLUTIL_F( char * )
lutil_SHA1File
	PORT_P((char *, char *));

LIBLUTIL_F( char * )
lutil_SHA1Data
	PORT_P((const unsigned char *, size_t, char *));

PORT_END_DECL

#endif /* _LUTIL_SHA1_H_ */
