/* $OpenLDAP: pkg/ldap/include/port_pvt_thread.h,v 1.19 1999/11/27 22:40:08 hyc Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */
/* port_pvt_thread.h - PortThreads threads header file NG */


/*
	PortThreads is derived from the portability layer of the OpenLDAP project.
	this is an attempt to make it more generalized and useful to other apps.
*/

#ifndef _PORT_PVT_THREAD_H
#define _PORT_PVT_THREAD_H

#include "port_threads_portable.h"
#include "port_threads_cdefs.h"

#if defined( HAVE_PTHREADS )
/**********************************
 *                                *
 * definitions for POSIX Threads  *
 *                                *
 **********************************/

#define __need_timespec
#include <time.h>

#include <sys/types.h>

#include <pthread.h>

#ifdef HAVE_SCHED_H
#include <sched.h>
#endif

PORT_BEGIN_DECL

#if !defined(HAVE_START_ROUTINE)

#define HAVE_START_ROUTINE
typedef void *(*start_routine)( void * );
typedef void *(*PORT_THREADS_START_ROUTINE)( void * );

#endif /*!defined(HAVE_START_ROUTINE)*/

typedef pthread_t		 port_pvt_thread_t;
typedef pthread_mutex_t		port_pvt_thread_mutex_t;
typedef pthread_cond_t		port_pvt_thread_cond_t;
typedef pthread_condattr_t	port_pvt_thread_condattr_t;
/*, port_thread_condattr_t;*/

typedef pthread_attr_t          port_pvt_thread_attr_t;

#define HAVE_PTHREAD_ATTR_T


#if HAVE_PTHREADS_D4
#  define PORT_PVT_THREAD_ATTR_DEFAULT		pthread_attr_default
#  define PORT_PVT_THREAD_CONDATTR_DEFAULT	pthread_condattr_default
#  define PORT_PVT_THREAD_MUTEXATTR_DEFAULT	pthread_mutexattr_default
#else
#  define PORT_PVT_THREAD_ATTR_DEFAULT		NULL
#  define PORT_PVT_THREAD_CONDATTR_DEFAULT	NULL
#  define PORT_PVT_THREAD_MUTEXATTR_DEFAULT	NULL
#endif

#  define PORT_THREAD_ATTR_DEFAULT		PORT_PVT_THREAD_ATTR_DEFAULT
#  define PORT_THREAD_CONDATTR_DEFAULT	        PORT_PVT_THREAD_CONDATTR_DEFAULT
#  define PORT_THREAD_MUTEXATTR_DEFAULT	        PORT_PVT_THREAD_MUTEXATTR_DEFAULT

#if defined( _POSIX_REENTRANT_FUNCTIONS ) || \
	defined( _POSIX_THREAD_SAFE_FUNCTIONS ) || \
	defined( _POSIX_THREADSAFE_FUNCTIONS )
#define HAVE_REENTRANT_FUNCTIONS 1
#endif

#if defined( HAVE_PTHREAD_GETCONCURRENCY ) || \
	defined( HAVE_THR_GETCONCURRENCY )
#define HAVE_GETCONCURRENCY 1
#endif

#if defined( HAVE_PTHREAD_SETCONCURRENCY ) || \
	defined( HAVE_THR_SETCONCURRENCY )
#define HAVE_SETCONCURRENCY 1
#endif

PORT_END_DECL

#elif defined ( HAVE_MACH_CTHREADS )
/**********************************
 *                                *
 * definitions for Mach CThreads  *
 *                                *
 **********************************/

#include <mach/cthreads.h>

PORT_BEGIN_DECL

typedef cthread_t		port_pvt_thread_t;
typedef struct mutex		port_pvt_thread_mutex_t;
typedef struct condition	port_pvt_thread_cond_t;

PORT_END_DECL

#elif defined( HAVE_GNU_PTH )
/***********************************
 *                                 *
 * thread definitions for GNU Pth  *
 *                                 *
 ***********************************/

#include <pth.h>

PORT_BEGIN_DECL

typedef pth_t		port_pvt_thread_t;
typedef pth_mutex_t	port_pvt_thread_mutex_t;
typedef pth_cond_t	port_pvt_thread_cond_t;

PORT_END_DECL


#elif defined( HAVE_THR )
/********************************************
 *                                          *
 * thread definitions for Solaris LWP (THR) *
 *                                          *
 ********************************************/

#include <thread.h>
#include <synch.h>

PORT_BEGIN_DECL

typedef thread_t		port_pvt_thread_t;
typedef mutex_t			port_pvt_thread_mutex_t;
typedef cond_t			port_pvt_thread_cond_t;
typedef pthread_attr_t          port_pvt_thread_attr_t;

#define HAVE_PTHREAD_ATTR_T
#define HAVE_REENTRANT_FUNCTIONS 1

#ifdef HAVE_THR_GETCONCURRENCY
#define HAVE_GETCONCURRENCY 1
#endif
#ifdef HAVE_THR_SETCONCURRENCY
#define HAVE_SETCONCURRENCY 1
#endif

PORT_END_DECL

#elif defined( HAVE_LWP )
/*************************************
 *                                   *
 * thread definitions for SunOS LWP  *
 *                                   *
 *************************************/

#include <lwp/lwp.h>
#include <lwp/stackdep.h>

PORT_BEGIN_DECL

typedef thread_t		port_pvt_thread_t;
typedef mon_t			port_pvt_thread_mutex_t;
struct port_pvt_thread_lwp_cv {
	int		lcv_created;
	cv_t		lcv_cv;
};
typedef struct port_pvt_thread_lwp_cv port_pvt_thread_cond_t;

#define HAVE_REENTRANT_FUNCTIONS 1

PORT_END_DECL

#elif defined(HAVE_NT_THREADS)

PORT_BEGIN_DECL

#include <process.h>
#include <windows.h>

typedef unsigned long	port_pvt_thread_t;
typedef HANDLE	port_pvt_thread_mutex_t;
typedef HANDLE	port_pvt_thread_cond_t;

PORT_END_DECL

#else

/***********************************
 *                                 *
 * thread definitions for no       *
 * underlying library support      *
 *                                 *
 ***********************************/

PORT_BEGIN_DECL

#ifndef NO_THREADS
#define NO_THREADS 1
#endif

#error Your threads a broken, or unsupported

typedef int			port_pvt_thread_t;
typedef int			port_pvt_thread_mutex_t;
typedef int			port_pvt_thread_cond_t;
typedef int			port_pvt_thread_condattr_t;

PORT_END_DECL

#endif /* no threads support */

PORT_BEGIN_DECL

LIBPORT_F( int )
port_pvt_thread_initialize PORT_P(( void ));

LIBPORT_F( int )
port_pvt_thread_destroy PORT_P(( void ));

LIBPORT_F( unsigned int )
port_pvt_thread_sleep PORT_P(( unsigned int s ));

#ifdef HAVE_GETCONCURRENCY
LIBPORT_F( int )
port_pvt_thread_get_concurrency PORT_P(( void ));
#endif

#ifdef HAVE_SETCONCURRENCY
#	ifndef PORT_THREAD_CONCURRENCY
	/* three concurrent threads should be enough */
#	define PORT_THREAD_CONCURRENCY	3
#	endif
LIBPORT_F( int )
port_pvt_thread_set_concurrency PORT_P(( int ));
#endif

#define PORT_PVT_THREAD_CREATE_JOINABLE 0
#define PORT_PVT_THREAD_CREATE_DETACHED 1

LIBPORT_F( int ) 
port_pvt_thread_create PORT_P((
	port_pvt_thread_t * thread, 
	int	detach,
	void *(*routine)( void * ),
	void *arg));

LIBPORT_F( void ) 
port_pvt_thread_exit PORT_P(( void *retval ));

LIBPORT_F( int )
port_pvt_thread_join PORT_P(( port_pvt_thread_t thread, void **status ));

LIBPORT_F( int )
port_pvt_thread_kill PORT_P(( port_pvt_thread_t thread, int signo ));

LIBPORT_F( int )
port_pvt_thread_yield PORT_P(( void ));

#if defined(HAVE_PTHREAD_ATTR_T)
LIBPORT_F( int )
port_pvt_thread_attr_init PORT_P(( port_pvt_thread_attr_t *attr ));

LIBPORT_F( int )
port_pvt_thread_attr_destroy PORT_P(( port_pvt_thread_attr_t *attr ));

LIBPORT_F( int )
port_pvt_thread_attr_set_scope PORT_P(( port_pvt_thread_attr_t *attr, int scope ));

LIBPORT_F( int )
port_pvt_thread_attr_get_scope PORT_P(( port_pvt_thread_attr_t *attr, int *scope ));

LIBPORT_F( int )
port_pvt_thread_attr_set_detachstate PORT_P(( port_pvt_thread_attr_t *attr, int state ));

LIBPORT_F( int )
port_pvt_thread_attr_get_detachstate PORT_P(( port_pvt_thread_attr_t *attr, int *state ));

LIBPORT_F( int )
     port_pvt_thread_attr_set_stacksize PORT_P(( port_pvt_thread_attr_t *attr, size_t size ));

LIBPORT_F( int )
port_pvt_thread_attr_get_stacksize PORT_P(( port_pvt_thread_attr_t *attr, size_t *size ));

LIBPORT_F( int )
port_pvt_thread_attr_set_stackaddr PORT_P(( port_pvt_thread_attr_t *attr, void *addr ));

LIBPORT_F( int )
port_pvt_thread_attr_get_stackaddr PORT_P(( port_pvt_thread_attr_t *attr, void **addr ));

#if defined(HAVE_SCHED_POLICY)
LIBPORT_F( int )
port_pvt_thread_attr_set_schedpolicy PORT_P(( port_pvt_thread_attr_t *attr ));

LIBPORT_F( int )
port_pvt_thread_attr_get_schedpolicy PORT_P(( port_pvt_thread_attr_t *attr ));
#endif /*defined(HAVE_SCHED_POLICY)*/

#if defined(HAVE_INHERIT_SCHED)
LIBPORT_F( int )
port_pvt_thread_attr_set_inheritsched PORT_P(( port_pvt_thread_attr_t *attr ));

LIBPORT_F( int )
port_pvt_thread_attr_get_inheritsched PORT_P(( port_pvt_thread_attr_t *attr ));
#endif /*defined(HAVE_INHERIT_SCHED)*/

#if defined(HAVE_SCHED_PARAM)
LIBPORT_F( int )
port_pvt_thread_attr_set_schedparam PORT_P(( port_pvt_thread_t *thr, int policy, const struct sched_param *param ));

LIBPORT_F( int )
port_pvt_thread_attr_get_schedparam PORT_P(( port_pvt_thread_t *attr, int *policy, struct sched_param *param ));
#endif /*defined(HAVE_SCHED_PARAM)*/
#endif /*defined(HAVE_PTHREAD_ATTR_T)*/

LIBPORT_F( int )
port_pvt_thread_cond_init PORT_P(( port_pvt_thread_cond_t *cond, const port_pvt_thread_condattr_t *cond_attr ));

LIBPORT_F( int )
port_pvt_thread_cond_destroy PORT_P(( port_pvt_thread_cond_t *cond ));

LIBPORT_F( int )
port_pvt_thread_cond_signal PORT_P(( port_pvt_thread_cond_t *cond ));

LIBPORT_F( int )
port_pvt_thread_cond_broadcast PORT_P(( port_pvt_thread_cond_t *cond ));

LIBPORT_F( int )
port_pvt_thread_cond_wait PORT_P((
	port_pvt_thread_cond_t *cond, 
	port_pvt_thread_mutex_t *mutex ));

LIBPORT_F( int )
port_pvt_thread_cond_timedwait PORT_P((
	port_pvt_thread_cond_t *cond, 
	port_pvt_thread_mutex_t *mutex,
	const struct timespec *time));

LIBPORT_F( int )
port_pvt_thread_condattr_init PORT_P(( port_pvt_thread_condattr_t *attr ));

LIBPORT_F( int )
port_pvt_thread_condattr_destroy PORT_P(( port_pvt_thread_condattr_t *attr ));

LIBPORT_F( int )
port_pvt_thread_mutex_init PORT_P(( port_pvt_thread_mutex_t *mutex ));

LIBPORT_F( int )
port_pvt_thread_mutex_destroy PORT_P(( port_pvt_thread_mutex_t *mutex ));

LIBPORT_F( int )
port_pvt_thread_mutex_lock PORT_P(( port_pvt_thread_mutex_t *mutex ));

LIBPORT_F( int )
port_pvt_thread_mutex_trylock PORT_P(( port_pvt_thread_mutex_t *mutex ));

LIBPORT_F( int )
port_pvt_thread_mutex_unlock PORT_P(( port_pvt_thread_mutex_t *mutex ));

typedef struct port_pvt_thread_rdwr_var {
	port_pvt_thread_mutex_t ltrw_mutex;	
	port_pvt_thread_cond_t ltrw_read;	/* wait for read */
	port_pvt_thread_cond_t ltrw_write;	/* wait for write */
	int ltrw_valid;
#define PORT_PVT_THREAD_RDWR_VALID 0x0bad
	int ltrw_r_active;
	int ltrw_w_active;
	int ltrw_r_wait;
	int ltrw_w_wait;
} port_pvt_thread_rdwr_t;

LIBPORT_F( int )
port_pvt_thread_rdwr_init PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_destroy PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_rlock PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_rtrylock PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_runlock PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_wlock PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_wtrylock PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_wunlock PORT_P((port_pvt_thread_rdwr_t *rdwrp));

#ifdef PORT_DEBUG
LIBPORT_F( int )
port_pvt_thread_rdwr_readers PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_writers PORT_P((port_pvt_thread_rdwr_t *rdwrp));

LIBPORT_F( int )
port_pvt_thread_rdwr_active PORT_P((port_pvt_thread_rdwr_t *rdwrp));
#endif /* PORT_DEBUG */

#define PORT_PVT_THREAD_EINVAL EINVAL
#define PORT_PVT_THREAD_EBUSY EINVAL

PORT_END_DECL

#endif /* _PORT_THREAD_H */
