/*
  Derived from the beos port_threads port.
 */
#include "port_threads.h"
#include <unistd.h>
#include <stdio.h>


void* thread_func1(void *data);
void* thread_func2(void *data);
void* thread_func3(void *data);
void* thread_func4(void *data);


port_thread_mutex_t counter_lock;
int counter=0;

port_thread_cond_t sent_signal_cond;
port_thread_mutex_t sent_signal_cond_lock;
int sent_signal=0;


void* thread_func(void *data)
{
    int i;
    printf("Inside %s:\n",__FUNCTION__);
    printf("  Acquiring sent_signal_cond_lock.\n");
    port_thread_mutex_lock(&sent_signal_cond_lock);
    printf("  sent_signal_cond_lock acquired.\n");
    printf("  Waiting on sent_signal_cond.\n");
    while(!sent_signal)
    {
      port_thread_cond_wait(&sent_signal_cond,&sent_signal_cond_lock);
    }
    printf("  Releasing sent_signal_cond_lock.\n");
    port_thread_mutex_unlock(&sent_signal_cond_lock);
    printf("  sent_signal_cond_lock released.\n");

    for (i = 0; i < 10000; i++) {
        port_thread_mutex_lock(&counter_lock);
        counter++;
        port_thread_mutex_unlock(&counter_lock);
    }
    return NULL;
} 

int main()
{
    port_thread_t t2;
    port_thread_t t3;
    port_thread_t t4;
    
    int s1=0;
    int s2=0;
    int s3=0;
    int s4=0;
    int i=0;
	
    fprintf(stdout, "Initializing the locks................"); 
    s1 = port_thread_mutex_init(&counter_lock); 
    if (s1 != 0) {
        fprintf(stderr, "Could not create lock\n");
        exit(-1);
    }
    s1 = port_thread_mutex_init(&sent_signal_cond_lock); 
    if (s1 != 0) {
        fprintf(stderr, "Could not create lock\n");
        exit(-1);
    }
    fprintf(stdout, "OK\n");

    fprintf(stdout,"Creating the conditional variable.....");
    s1 = port_thread_cond_init(&sent_signal_cond,PORT_THREAD_CONDATTR_DEFAULT);
    if (s1 != 0) {
        fprintf(stderr, "Could not create conditional\n");
        exit(-1);
    }
    fprintf(stdout, "OK\n");
	
    fprintf(stdout, "Starting all the threads.............."); 
    s2 = port_thread_create(&t2, 0, thread_func, NULL);
    s3 = port_thread_create(&t3, 0, thread_func, NULL);
    s4 = port_thread_create(&t4, 0, thread_func, NULL);
    if (s1 != 0 || s2 != 0 || 
        s3 != 0 || s4 != 0) {
        fprintf(stderr, "Error starting thread\n");
        exit(-1);
    }
    fprintf(stdout, "OK\n");
	
    fprintf (stdout,"Trying to signal conditional..........");
    fflush(stdout);
    port_thread_mutex_lock(&sent_signal_cond_lock);
    sent_signal=1;
    port_thread_mutex_unlock(&sent_signal_cond_lock);

    if (port_thread_cond_signal(&sent_signal_cond) != 0)
      fprintf (stdout,"failed!\n");
    else
      fprintf(stdout,"OK\n");

    fprintf (stdout,"Trying to broadcast conditional.......");
    if (port_thread_cond_broadcast(&sent_signal_cond) != 0)
      fprintf (stdout,"failed!\n");
    else
      fprintf(stdout,"OK\n");

    fprintf(stdout, "Waiting for threads to exit...........");
    fflush(stdout);
    port_thread_join(t2, NULL);
    port_thread_join(t3, NULL);
    port_thread_join(t4, NULL);
    fprintf (stdout, "OK\n");   

    fprintf(stdout, "Checking if locks worked..............");
    if (counter != 30000) {
      fprintf(stderr, "The locks didn't work????  %d\n", counter);
    }
    else {
      fprintf(stdout, "Everything is working!\n");
    }

    fprintf(stdout,"Destroying port_thread variables used.\n");
    port_thread_mutex_destroy(&sent_signal_cond_lock);
    port_thread_mutex_destroy(&counter_lock);
    port_thread_cond_destroy(&sent_signal_cond);	
    return 1;
}
