/* $OpenLDAP: pkg/ldap/libraries/libport_r/thr_cthreads.c,v 1.8 1999/09/08 17:06:30 kdz Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */

/* thr_cthreads.c - wrapper for mach cthreads */

/*
	PortThreads is derived from the portability layer of the OpenLDAP project.
	this is an attempt to make it more generalized and useful to other apps.
*/

#include "port_threads_portable.h"

#if defined( HAVE_MACH_CTHREADS )
#include "port_pvt_thread.h"

/***********************************************************************
 *                                                                     *
 * under NEXTSTEP or OPENSTEP use CThreads                             *
 * lukeh@xedoc.com.au                                                  *
 *                                                                     *
 ***********************************************************************/

int
port_pvt_thread_initialize( void )
{
	return 0;
}

int
port_pvt_thread_destroy( void )
{
	return 0;
}

int 
port_pvt_thread_create( port_pvt_thread_t * thread, 
	int detach,
	start_routine routine,
	void *arg)
{
	*thread = cthread_fork( (cthread_fn_t) routine, arg);
	return ( *thread == NULL ? -1 : 0 );	
}

void 
port_pvt_thread_exit( void *retval )
{
	cthread_exit( (any_t) retval );
}

int 
port_pvt_thread_join( port_pvt_thread_t thread, void **thread_return )
{
	void *status;
	status = (void *) cthread_join ( thread );
	if (thread_return != NULL)
		{
		*thread_return = status;
		}
	return 0;
}

int 
port_pvt_thread_kill( port_pvt_thread_t thread, int signo )
{
	return 0;
}

int 
port_pvt_thread_yield( void )
{
	cthread_yield();
	return 0;
}

int 
port_pvt_thread_cond_init( port_pvt_thread_cond_t *cond )
{
	condition_init( cond );
	return( 0 );
}

int 
port_pvt_thread_cond_destroy( port_pvt_thread_cond_t *cond )
{
	condition_clear( cond );
	return( 0 );
}

int 
port_pvt_thread_cond_signal( port_pvt_thread_cond_t *cond )
{
	condition_signal( cond );
	return( 0 );
}

int
port_pvt_thread_cond_broadcast( port_pvt_thread_cond_t *cond )
{
	condition_broadcast( cond );
	return( 0 );
}

int 
port_pvt_thread_cond_wait( port_pvt_thread_cond_t *cond, 
			  port_pvt_thread_mutex_t *mutex )
{
	condition_wait( cond, mutex );
	return( 0 );	
}

int 
port_pvt_thread_mutex_init( port_pvt_thread_mutex_t *mutex )
{
	mutex_init( mutex );
	mutex->name = NULL;
	return ( 0 );
}

int 
port_pvt_thread_mutex_destroy( port_pvt_thread_mutex_t *mutex )
{
	mutex_clear( mutex );
	return ( 0 );	
}
	
int 
port_pvt_thread_mutex_lock( port_pvt_thread_mutex_t *mutex )
{
	mutex_lock( mutex );
	return ( 0 );
}

int 
port_pvt_thread_mutex_unlock( port_pvt_thread_mutex_t *mutex )
{
	mutex_unlock( mutex );
	return ( 0 );
}

int
port_pvt_thread_mutex_trylock( port_pvt_thread_mutex_t *mutex )
{
	return mutex_try_lock( mutex );
}

#endif /* HAVE_MACH_CTHREADS */
