/* $OpenLDAP: pkg/ldap/libraries/libport_r/thr_nt.c,v 1.9 1999/09/08 17:06:30 kdz Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */

/* thr_nt.c - wrapper around NT threads */

/*
	PortThreads is derived from the portability layer of the OpenLDAP project.
	this is an attempt to make it more generalized and useful to other apps.
*/

#include "port_threads_portable.h"

#if defined( HAVE_NT_THREADS )

#include "port_pvt_thread.h"

int
port_pvt_thread_initialize( void )
{
	return 0;
}

int
port_pvt_thread_destroy( void )
{
	return 0;
}

int 
port_pvt_thread_create( port_pvt_thread_t * thread, 
	int detach,
	start_routine routine,
	void *arg)
{
	*thread = (port_pvt_thread_t)_beginthread( (void *) routine, 
						0, arg );
	 return ( (unsigned long)*thread == -1 ? -1 : 0 );
}
	
void 
port_pvt_thread_exit( void *retval )
{
	_endthread( );
}

int 
port_pvt_thread_join( port_pvt_thread_t thread, void **thread_return )
{
	DWORD status;
	status = WaitForSingleObject( (HANDLE) thread, INFINITE );
	if (status == WAIT_FAILED) {
		return -1;
	}
	return 0;
}

int 
port_pvt_thread_kill( port_pvt_thread_t thread, int signo )
{
	return 0;
}

int 
port_pvt_thread_yield( void )
{
	Sleep( 0 );
	return 0;
}

int 
port_pvt_thread_cond_init( port_pvt_thread_cond_t *cond )
{
	*cond = CreateEvent( NULL, FALSE, FALSE, NULL );
	return( 0 );
}

int
port_pvt_thread_cond_destroy( port_pvt_thread_cond_t *cv )
{
	CloseHandle( *cv );
	return( 0 );
}

int 
port_pvt_thread_cond_signal( port_pvt_thread_cond_t *cond )
{
	SetEvent( *cond );
	return( 0 );
}

int 
port_pvt_thread_cond_wait( port_pvt_thread_cond_t *cond, 
			  port_pvt_thread_mutex_t *mutex )
{
	ReleaseMutex( *mutex );
	WaitForSingleObject( *cond, INFINITE );
	WaitForSingleObject( *mutex, INFINITE );
	return( 0 );
}

int
port_pvt_thread_cond_broadcast( port_pvt_thread_cond_t *cv )
{
	SetEvent( *cv );
	return( 0 );
}

int 
port_pvt_thread_mutex_init( port_pvt_thread_mutex_t *mutex )
{
	*mutex = CreateMutex( NULL, 0, NULL );
	return ( 0 );
}

int 
port_pvt_thread_mutex_destroy( port_pvt_thread_mutex_t *mutex )
{
	CloseHandle( *mutex );
	return ( 0 );	
}

int 
port_pvt_thread_mutex_lock( port_pvt_thread_mutex_t *mutex )
{
	WaitForSingleObject( *mutex, INFINITE );
	return ( 0 );
}

int 
port_pvt_thread_mutex_unlock( port_pvt_thread_mutex_t *mutex )
{
	ReleaseMutex( *mutex );
	return ( 0 );
}

int
port_pvt_thread_mutex_trylock( port_pvt_thread_mutex_t *mp )
{
	DWORD status;

	status = WaitForSingleObject( *mp, 0 );
	if ( (status == WAIT_FAILED) || (status == WAIT_TIMEOUT) )
		return 0;
	else
		return 1;
}

#endif
