/* $OpenLDAP: pkg/ldap/libraries/libport_r/thr_posix.c,v 1.12 1999/11/28 03:07:11 hyc Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */

/* thr_posix.c - wrapper around posix and posixish thread implementations.
 */

/*
	PortThreads is derived from the portability layer of the OpenLDAP project.
	this is an attempt to make it more generalized and useful to other apps.
*/

#include "port_threads_portable.h"

#if defined( HAVE_PTHREADS )

#include <ac/errno.h>

#include "port_pvt_thread.h"


#if HAVE_PTHREADS_D4
#  define PORT_PVT_THREAD_ATTR_DEFAULT		pthread_attr_default
#  define PORT_PVT_THREAD_CONDATTR_DEFAULT	pthread_condattr_default
#  define PORT_PVT_THREAD_MUTEXATTR_DEFAULT	pthread_mutexattr_default
#else
#  define PORT_PVT_THREAD_ATTR_DEFAULT		NULL
#  define PORT_PVT_THREAD_CONDATTR_DEFAULT	NULL
#  define PORT_PVT_THREAD_MUTEXATTR_DEFAULT	NULL
#endif



LIBPORT_IF(int)
port_pvt_thread_initialize( void )
{
#if defined( PORT_THREAD_CONCURRENCY ) && HAVE_PTHREAD_SETCONCURRENCY
	port_pvt_thread_set_concurrency( PORT_THREAD_CONCURRENCY );
#endif
	return 0;
}


LIBPORT_IF(int)
port_pvt_thread_destroy( void )
{
	return 0;
}

#if defined(HAVE_PTHREAD_SETCONCURRENCY) || defined(HAVE_THR_SETCONCURRENCY)
LIBPORT_IF(int)
port_pvt_thread_set_concurrency(int n)
{
#ifdef HAVE_PTHREAD_SETCONCURRENCY
	return pthread_setconcurrency( n );
#elif HAVE_THR_SETCONCURRENCY
	return thr_setconcurrency( n );
#else
	return 0;
#endif
}
#endif


#if defined(HAVE_PTHREAD_GETCONCURRENCY) || defined(HAVE_THR_GETCONCURRENCY)
LIBPORT_IF(int)
port_pvt_thread_get_concurrency(void)
{
#ifdef HAVE_PTHREAD_GETCONCURRENCY
	return pthread_getconcurrency();
#elif HAVE_THR_GETCONCURRENCY
	return thr_getconcurrency();
#else
	return 0;
#endif
}
#endif

LIBPORT_IF(int) 
port_pvt_thread_create( port_pvt_thread_t * thread,
	int detach,
	start_routine routine,
	void *arg)
{
	int rtn = pthread_create( thread, PORT_PVT_THREAD_ATTR_DEFAULT,
				  routine, arg );

	if( detach ) {
#ifdef HAVE_PTHREADS_FINAL
		pthread_detach( *thread );
#else
		pthread_detach( thread );
#endif
	}
	return rtn;
}

LIBPORT_IF(void)
port_pvt_thread_exit( void *retval )
{
	pthread_exit( retval );
}

LIBPORT_IF(int) 
port_pvt_thread_join( port_pvt_thread_t thread, void **thread_return )
{
#if !defined( HAVE_PTHREADS_FINAL )
	void *dummy;
	if (thread_return==NULL)
	  thread_return=&dummy;
#endif	
	return pthread_join( thread, thread_return );
}

LIBPORT_IF(int) 
port_pvt_thread_kill( port_pvt_thread_t thread, int signo )
{
#ifdef HAVE_PTHREAD_KILL
	return pthread_kill( thread, signo );
#else
	/* pthread package with DCE */
	if (kill( getpid(), signo )<0)
		return errno;
	return 0;
#endif
}

LIBPORT_IF(int) 
port_pvt_thread_yield( void )
{
#ifdef _POSIX_THREAD_IS_GNU_PTH
	sched_yield();
	return 0;

#elif HAVE_SCHED_YIELD
	return sched_yield();

#elif HAVE_PTHREAD_YIELD
	pthread_yield();
	return 0;

#elif HAVE_THR_YIELD
	return thr_yield();

#else
	return 0;
#endif   
}

#if defined(HAVE_PTHREAD_ATTR_T)

LIBPORT_IF(int)
port_pvt_thread_attr_init PORT_P(( port_pvt_thread_attr_t *attr ))
{
	return pthread_attr_init (attr);
}

LIBPORT_IF(int)
port_pvt_thread_attr_destroy PORT_P(( port_pvt_thread_attr_t *attr ))
{
	return pthread_attr_destroy (attr);
}

LIBPORT_IF(int)
port_pvt_thread_attr_set_scope PORT_P(( port_pvt_thread_attr_t *attr, int scope ))
{
	return pthread_attr_setscope (attr,scope);
}

LIBPORT_IF(int)
port_pvt_thread_attr_get_scope PORT_P(( port_pvt_thread_attr_t *attr, int *scope ))
{
	return pthread_attr_getscope (attr,scope);
}

LIBPORT_IF(int)
port_pvt_thread_attr_set_detachstate PORT_P(( port_pvt_thread_attr_t *attr, int state ))
{
	return pthread_attr_setdetachstate (attr,state);
}

LIBPORT_IF(int)
port_pvt_thread_attr_get_detachstate PORT_P(( port_pvt_thread_attr_t *attr, int *state ))
{
	return pthread_attr_getdetachstate (attr,state);
}

LIBPORT_IF(int)
port_pvt_thread_attr_set_stacksize PORT_P(( port_pvt_thread_attr_t *attr, size_t size ))
{
	return pthread_attr_setstacksize (attr,size);
}

LIBPORT_IF(int)
port_pvt_thread_attr_get_stacksize PORT_P(( port_pvt_thread_attr_t *attr, size_t *size ))
{
	return pthread_attr_getstacksize (attr,size);
}

LIBPORT_IF(int)
port_pvt_thread_attr_set_stackaddr PORT_P(( port_pvt_thread_attr_t *attr, void *addr ))
{
	return pthread_attr_setstackaddr (attr,addr);
}

LIBPORT_IF(int)
port_pvt_thread_attr_get_stackaddr PORT_P(( port_pvt_thread_attr_t *attr, void **addr ))
{
	return pthread_attr_getstackaddr (attr,addr);
}

#if defined(HAVE_SCHED_POLICY)
LIBPORT_IF(int)
port_pvt_thread_attr_set_schedpolicy PORT_P(( port_pvt_thread_attr_t *attr ))
{
	return pthread_attr_setschedpolicy (attr);
}

LIBPORT_IF(int)
port_pvt_thread_attr_get_schedpolicy PORT_P(( port_pvt_thread_attr_t *attr, in *policy))
{
	return pthread_attr_getschedpolicy (attr,policy);
}
#endif /*defined(HAVE_SCHED_POLICY)*/

#if defined(HAVE_INHERIT_SCHED)
LIBPORT_IF(int)
port_pvt_thread_attr_set_inheritsched PORT_P(( port_pvt_thread_attr_t *attr, int inheritsched ))
{
	return pthread_attr_setinheritsched (attr,inheritsched);
}

LIBPORT_IF(int)
port_pvt_thread_attr_get_inheritsched PORT_P(( port_pvt_thread_attr_t *attr, int *inheritsched))
{
	return pthread_attr_getinheritsched (attr,inheritsched);
}
#endif /*defined(HAVE_INHERIT_SCHED)*/

#if defined(HAVE_SCHED_PARAM)
LIBPORT_IF(int)
port_pvt_thread_attr_set_schedparam PORT_P(( port_pvt_thread_t *thr, int policy, const struct sched_param *param ))
{
	return pthread_attr_setschedparam (thr,policy,param);
}

LIBPORT_IF(int)
port_pvt_thread_attr_get_schedparam PORT_P(( port_pvt_thread_t *attr, int *policy, struct sched_param *param ))
{
	return pthread_attr_getschedparam (thr,policy,param);
}
#endif /*idefined(HAVE_SCHED_PARAM)*/

#endif /*defined(HAVE_PTHREAD_ATTR_T)*/

LIBPORT_IF(int) 
port_pvt_thread_cond_init( port_pvt_thread_cond_t *cond, const port_pvt_thread_condattr_t *cond_attr )
{
	return pthread_cond_init( cond, cond_attr);
}

LIBPORT_IF(int) 
port_pvt_thread_cond_destroy( port_pvt_thread_cond_t *cond )
{
	return pthread_cond_destroy( cond );
}
	
LIBPORT_IF(int)
port_pvt_thread_cond_signal( port_pvt_thread_cond_t *cond )
{
	return pthread_cond_signal( cond );
}

LIBPORT_IF(int)
port_pvt_thread_cond_broadcast( port_pvt_thread_cond_t *cond )
{
	return pthread_cond_broadcast( cond );
}

LIBPORT_IF(int)
port_pvt_thread_cond_wait( port_pvt_thread_cond_t *cond, 
		      port_pvt_thread_mutex_t *mutex )
{
	return pthread_cond_wait( cond, mutex );
}

LIBPORT_IF(int)
port_pvt_thread_mutex_init( port_pvt_thread_mutex_t *mutex )
{
	return pthread_mutex_init( mutex, PORT_PVT_THREAD_MUTEXATTR_DEFAULT );
}

LIBPORT_IF(int)
port_pvt_thread_mutex_destroy( port_pvt_thread_mutex_t *mutex )
{
	return pthread_mutex_destroy( mutex );
}

LIBPORT_IF(int)
port_pvt_thread_mutex_lock( port_pvt_thread_mutex_t *mutex )
{
	return pthread_mutex_lock( mutex );
}

LIBPORT_IF(int)
port_pvt_thread_mutex_trylock( port_pvt_thread_mutex_t *mutex )
{
	return pthread_mutex_trylock( mutex );
}

LIBPORT_IF(int)
port_pvt_thread_mutex_unlock( port_pvt_thread_mutex_t *mutex )
{
	return pthread_mutex_unlock( mutex );
}

#endif /* HAVE_PTHREADS */

