/* $OpenLDAP: pkg/ldap/libraries/libport_r/thr_pth.c,v 1.2 1999/09/08 17:06:30 kdz Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */

/* thr_pth.c - wrappers around GNU Pth threads */

/*
	PortThreads is derived from the portability layer of the OpenLDAP project.
	this is an attempt to make it more generalized and useful to other apps.
*/

#include "port_threads_portable.h"

#if defined( HAVE_GNU_PTH )

#include "port_pvt_thread.h"

/*******************
 *                 *
 * GNU Pth Threads *
 *                 *
 *******************/

static pth_attr_t detach_attr;

int
port_pvt_thread_initialize( void )
{
	detach_attr = pth_attr_new();
	pth_attr_set( detach_attr, PTH_ATTR_JOINABLE, FALSE );
	return pth_init();
}

int
port_pvt_thread_destroy( void )
{
	pth_attr_destroy(detach_attr);
	pth_kill();
	return 0;
}

int 
port_pvt_thread_create( port_pvt_thread_t * thread, 
	int detach,
	void *(*start_routine)( void *),
	void *arg)
{
	*thread = pth_spawn( detach ? detach_attr : PTH_ATTR_DEFAULT,
		start_routine, arg );

	return *thread == NULL;
}

void 
port_pvt_thread_exit( void *retval )
{
	pth_exit( retval );
}

int port_pvt_thread_join( port_pvt_thread_t thread, void **thread_return )
{
	pth_join( thread, thread_return );
	return 0;
}

int 
port_pvt_thread_kill( port_pvt_thread_t thread, int signo )
{
	pth_raise( thread, signo );
	return 0;
}
	
int 
port_pvt_thread_yield( void )
{
	pth_yield(NULL);
	return 0;
}

int 
port_pvt_thread_cond_init( port_pvt_thread_cond_t *cond )
{
	return( pth_cond_init( cond ) );
}

int 
port_pvt_thread_cond_signal( port_pvt_thread_cond_t *cond )
{
	return( pth_cond_notify( cond, 0 ) );
}

int
port_pvt_thread_cond_broadcast( port_pvt_thread_cond_t *cond )
{
	return( pth_cond_notify( cond, 1 ) );
}

int 
port_pvt_thread_cond_wait( port_pvt_thread_cond_t *cond, 
			  port_pvt_thread_mutex_t *mutex )
{
	return( pth_cond_await( cond, mutex, NULL ) );
}

int
port_pvt_thread_cond_destroy( port_pvt_thread_cond_t *cv )
{
	return 0;
}

int 
port_pvt_thread_mutex_init( port_pvt_thread_mutex_t *mutex )
{
	return( pth_mutex_init( mutex ) );
}

int 
port_pvt_thread_mutex_destroy( port_pvt_thread_mutex_t *mutex )
{
	return 0;
}

int 
port_pvt_thread_mutex_lock( port_pvt_thread_mutex_t *mutex )
{
	return( pth_mutex_acquire( mutex, 0, NULL ) );
}

int 
port_pvt_thread_mutex_unlock( port_pvt_thread_mutex_t *mutex )
{
	return( pth_mutex_release( mutex ) );
}

int
port_pvt_thread_mutex_trylock( port_pvt_thread_mutex_t *mutex )
{
	return( pth_mutex_acquire( mutex, 1, NULL ) );
}

#endif /* HAVE_GNU_PTH */
