/* $OpenLDAP: pkg/ldap/libraries/libport_r/thr_stub.c,v 1.8 1999/09/08 17:06:30 kdz Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */

/* thr_stub.c - stubs for the threads */

/*
	PortThreads is derived from the portability layer of the OpenLDAP project.
	this is an attempt to make it more generalized and useful to other apps.
*/

#include "port_threads_portable.h"

#if defined( NO_THREADS )

#include "port_pvt_thread.h"

/***********************************************************************
 *                                                                     *
 * no threads package defined for this system - fake ok returns from   *
 * all threads routines (making it single-threaded).                   *
 *                                                                     *
 ***********************************************************************/

int
port_pvt_thread_initialize( void )
{
	return 0;
}

int
port_pvt_thread_destroy( void )
{
	return 0;
}

static void* port_int_status = NULL;

int 
port_pvt_thread_create( port_pvt_thread_t * thread, 
	int detach,
	start_routine routine,
	void *arg)
{
	if( ! detach ) port_int_status = NULL;
	routine( arg );
	return 0;
}

void 
port_pvt_thread_exit( void *retval )
{
	if( retval != NULL ) {
		port_int_status = retval;
	}
	return;
}

int 
port_pvt_thread_join( port_pvt_thread_t thread, void **status )
{
	if(status != NULL) *status = port_int_status;
	return 0;
}

int 
port_pvt_thread_kill( port_pvt_thread_t thread, int signo )
{
	return 0;
}

int 
port_pvt_thread_yield( void )
{
	return 0;
}

int 
port_pvt_thread_cond_init( port_pvt_thread_cond_t *cond )
{
	return 0;
}

int 
port_pvt_thread_cond_destroy( port_pvt_thread_cond_t *cond )
{
	return 0;
}

int 
port_pvt_thread_cond_signal( port_pvt_thread_cond_t *cond )
{
	return 0;
}

int 
port_pvt_thread_cond_broadcast( port_pvt_thread_cond_t *cond )
{
	return 0;
}

int 
port_pvt_thread_cond_wait( port_pvt_thread_cond_t *cond,
			  port_pvt_thread_mutex_t *mutex )
{
	return 0;
}

int 
port_pvt_thread_mutex_init( port_pvt_thread_mutex_t *mutex )
{
	return 0;
}

int 
port_pvt_thread_mutex_destroy( port_pvt_thread_mutex_t *mutex )
{
	return 0;
}

int 
port_pvt_thread_mutex_lock( port_pvt_thread_mutex_t *mutex )
{
	return 0;
}

int 
port_pvt_thread_mutex_unlock( port_pvt_thread_mutex_t *mutex )
{
	return 0;
}

#endif /* NO_THREADS */
