/* $OpenLDAP: pkg/ldap/libraries/libport_r/thr_thr.c,v 1.6 1999/09/08 17:06:30 kdz Exp $ */
/*
 * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted only
 * as authorized by the OpenLDAP Public License.  A copy of this
 * license is available at http://www.OpenLDAP.org/license.html or
 * in file LICENSE in the top-level directory of the distribution.
 */

/* thr_thr.c - wrappers around solaris threads */

/*
	PortThreads is derived from the portability layer of the OpenLDAP project.
	this is an attempt to make it more generalized and useful to other apps.
*/

#include "port_threads_portable.h"

#if defined( HAVE_THR )

#include "port_pvt_thread.h"

/*******************
 *                 *
 * Solaris Threads *
 *                 *
 *******************/

int
port_pvt_thread_initialize( void )
{
#ifdef PORT_THREAD_CONCURRENCY
	thr_setconcurrency( PORT_THREAD_CONCURRENCY );
#endif
	return 0;
}

int
port_pvt_thread_destroy( void )
{
	return 0;
}

int
port_pvt_thread_set_concurrency(int n)
{
	return thr_setconcurrency( n );
}

int
port_pvt_thread_get_concurrency(void)
{
	return thr_getconcurrency();
}

int 
port_pvt_thread_create( port_pvt_thread_t * thread, 
	int detach,
	start_routine routine,
	void *arg)
{
	return( thr_create( NULL, 0, routine, arg,
		detach ? THR_DETACHED : 0,
		thread ) );
}

void 
port_pvt_thread_exit( void *retval )
{
	thr_exit( NULL );
}

int port_pvt_thread_join( port_pvt_thread_t thread, void **thread_return )
{
	thr_join( thread, NULL, thread_return );
	return 0;
}

int 
port_pvt_thread_kill( port_pvt_thread_t thread, int signo )
{
	thr_kill( thread, signo );
	return 0;
}
	
int 
port_pvt_thread_yield( void )
{
	thr_yield();
	return 0;
}

int 
port_pvt_thread_cond_init( port_pvt_thread_cond_t *cond )
{
	return( cond_init( cond, USYNC_THREAD, NULL ) );
}

int 
port_pvt_thread_cond_signal( port_pvt_thread_cond_t *cond )
{
	return( cond_signal( cond ) );
}

int
port_pvt_thread_cond_broadcast( port_pvt_thread_cond_t *cv )
{
	return( cond_broadcast( cv ) );
}

int 
port_pvt_thread_cond_wait( port_pvt_thread_cond_t *cond, 
			  port_pvt_thread_mutex_t *mutex )
{
	return( cond_wait( cond, mutex ) );
}

int
port_pvt_thread_cond_destroy( port_pvt_thread_cond_t *cv )
{
	return( cond_destroy( cv ) );
}

int 
port_pvt_thread_mutex_init( port_pvt_thread_mutex_t *mutex )
{
	return( mutex_init( mutex, USYNC_THREAD, NULL ) );
}

int 
port_pvt_thread_mutex_destroy( port_pvt_thread_mutex_t *mutex )
{
	return( mutex_destroy( mutex ) );
}

int 
port_pvt_thread_mutex_lock( port_pvt_thread_mutex_t *mutex )
{
	return( mutex_lock( mutex ) );
}

int 
port_pvt_thread_mutex_unlock( port_pvt_thread_mutex_t *mutex )
{
	return( mutex_unlock( mutex ) );
}

int
port_pvt_thread_mutex_trylock( port_pvt_thread_mutex_t *mp )
{
	return( mutex_trylock( mp ) );
}

#endif /* HAVE_THR */
